package com.vromtr.tabbedmaps;

import java.util.List;


import android.app.Dialog;
import android.app.PendingIntent;
import android.content.Context;
import android.content.Intent;
import android.content.IntentFilter;
import android.content.res.Resources;
import android.location.Criteria;
import android.location.Location;
import android.location.LocationListener;
import android.location.LocationManager;
import android.os.Bundle;
import android.view.View;
import android.view.View.OnClickListener;
import android.widget.AdapterView;
import android.widget.ArrayAdapter;
import android.widget.Button;
import android.widget.ListView;
import android.widget.TabHost;
import android.widget.TextView;
import android.widget.AdapterView.OnItemClickListener;

import com.google.android.maps.GeoPoint;
import com.google.android.maps.MapActivity;
import com.google.android.maps.MapController;
import com.google.android.maps.MapView;
import com.google.android.maps.Overlay;

public class TabbedMaps extends MapActivity {

	//LocationManager is a class for talking with the GPS and other location hardware on the phone
	LocationManager mLocationManager;
	LocationListener mLocationListener;
	String mProvider;
	
	//The phone's actual location in meatspace, according to GPS
	Location mMyLocation;
	
	//The Google Maps control.
	private MapView mMapView;
	
	//Class used to interact with the Google Maps control
	private MapController mMapController;
	
	//"Overlays" are transparent drawing surfaces that go over a view
	//Our DynamicOverlay class handles annotating the map and creating informational dialogs
	private DynamicOverlay mMarkers;
	
	//A TabHost is used to control the TabView, the control that wraps around our subviews
	private TabHost mTabHost;
	
	//List of points of interest for Tab #2
	private static final String[] pointsOfInterest = new String[] { "My Location",  "Cofrin", "Boyle", "Todd Wehr Hall", "Mulva Library", "Bemis International Center", "Campus Center" };
	
	//Dialog constants used for instantiating the dialogs owned by this class
	private static final int DLG_BASE = 1;
	private static final int COFRIN_DIALOG = 	DLG_BASE + 0;
	private static final int BOYLE_DIALOG =		DLG_BASE + 1;
	private static final int WEHR_DIALOG = 		DLG_BASE + 2;
	private static final int MULVA_DIALOG = 	DLG_BASE + 3;
	private static final int BEMIS_DIALOG = 	DLG_BASE + 4;
	private static final int CC_DIALOG = 		DLG_BASE + 5;	//Campus Center, not shitsu from ko-do giasu
	
	
	//Points of interest listview on Tab #2
	private ListView mPoiList;
	
	
	/*
	 * Used for maintaining tour state
	 */
	
	private static int mTourState = 0;
	private static String[] mTourStops = { "Cofrin Hall", "Todd Wehr Hall", "Boyle Hall", "Bemis International Center", "Mulva Library", "Campus Center" };
	private static String[] mTourCaptions = {
		"First stop is Cofrin Hall!\n\nKeep in mind that the you can skip ahead in the tour or go back by pressing the arrow buttons at the top of this screen.\n\nThe Points of Interest tab contains a list of campus landmarks and pictures.\n\nThe Where Am I map contains a satellite view of campus.  The green arrow is the next destination on the tour.  Tap a map pushpin for more information on that landmark.",			//Cofrin
		"Now go to Todd Wehr Hall!",			//Todd
		"Now some meaningful and descriptive way of telling you to go to Boyle!",	//Boyle
		"Now go to the Bemis International Center!", //Bemis
		"Now go to the Mulva Library!", //Mulva
		"Now go to the Campus Center!"	//Campus Center
		};
	private static final int mTourNumStates = 6;
	
	/*
	 * Tour controls
	 */
	
	private TextView mTourCaption;
	private Button mTourPrev;
	private Button mTourNext;
	private Button mTourShowOnMap;
	
	/*
	 * Tour proximity sensing
	 */
	private Intent mTourProximityIntent;
	private PendingIntent mTourPendingIntent;
	
	
	
	/** Called when the activity is first created. */
    @Override
    public void onCreate(Bundle savedInstanceState) {
        //Call base method
    	super.onCreate(savedInstanceState);
        
    	//"Inflate" our layout
    	setContentView(R.layout.main);
    	
    	//Construct the tabs of our UI.  Must be done under program control.
    	setupTabHost();
    	
        //Construct the Points of Interest list on Tab #2
    	setupPointsOfInterestList();

        //Build and configure the MapView (Google Maps) control on tab #1
    	//setupGPS();
    	setupMapView();
    	
    	//Get our hook into our phone's GPS.  Use it to update our current location.
    	setupGPS();
    	

    	

    	
    	//Cofrin!
    	mMarkers.addNewItem(44.445809, -88.067335, R.drawable.cofrin, "Cofrin Hall", "Built 1992.  Total budget: $6 million.\n\nIn April 1988, the Fort Howard Corporation and David Cofrin, son of Austin E. Cofrin, the founder of Fort Howard, announced a naming gift of $1.5 million to the College for construction of a new classroom building named after his father.  Later that year, the Lynde and Harry Bradley Foundation of Milwaukee gave a gift of $400,000 toward the cost of the new academic building.\n\nThis four-level, 41,000 square foot building contains 10 classrooms and 30 faculty offices for the business administration, economics, computer science, and mathematics disciplines.  The Colleges mainframe and various microcomputer laboratories are also in the building.  The lower level contains a student lounge, a loading dock, and two lecture halls (75 seats and 150 seats), and an underground tunnel connecting Cofrin Hall with the Bemis International Center.\n\nCofrin Hall is located southeast of Pennings Activity Center on Reid Street, and was designed by CPR architectural firm of De Pere.  It is a masonry building of brick and limestone facing, compatible with other campus buildings.\n\nDr. David Cofrin retired in 1987 from his medical practice.  The Fort Howard foundation is a major philanthropic force in the community.\n\nGroundbreaking was held May 21, 1991 and dedication took place Sept. 11, 1992.");
    	
    	//Todd Wehr Hall
    	mMarkers.addNewItem(44.445017, -88.067571, R.drawable.todd, "Todd Wehr Hall", "St. Norbert Colleges first library was a small room on the first floor of Main Hall, which opened in 1903, and contained books donated by students. In 1924, the original recreation room of Boyle Hall became the main library until 1959, when the library was moved to Boyles third and fourth floors.\n\nIn 1977, the Todd Wehr Library, was established through the remodeling of an existing campus dormitory. The library, which is the center of student academic life on the campus, houses more than 247,000 volumes, including books, journals and other serials, microforms, maps and charts. Over 3,000 volumes are added to the collection each year.\n\nThe Todd Wehr has since been remodeled to house the registrar, payroll, and other campus departments along with the campus bookstore and the student-run Discoveries International.");
    	
    	//Boyle Hall
    	mMarkers.addNewItem(44.444702, -88.066418, R.drawable.boyle, "Boyle Hall", "The liberal arts building.  The first two floors are home to a number of smart classrooms featuring PCs and projectors; the upper two floors house faculty offices. \n\nThe basement is home to networking and telecommunications equipment as well as the grue - a sinister, lurking presence in the dark places of the earth. Its favorite diet is adventurers, but its insatiable appetite is tempered by its fear of light. No grue has ever been seen by the light of day, and few have survived its fearsome jaws to tell the tale.");
    	
    	//Bemis
    	mMarkers.addNewItem(44.445552, -88.067977, R.drawable.bemis, "Bemis International Center", "As you approach the St. Norbert Campus, the F.K. Bemis International Conference Center catches your eye.  This sophisticated facility plays host to weddings, conferences, dinners, and other gatherings for the college and community.  It features a state-of-the-art lecture theater that accommodates up to 200 people, eight multi-purpose rooms, computer labs and lounges.  Our Center for International Education houses our study abroad and ESL offices, and we offer language and translation services, along with adult and childrens foreign language classes for the non-traditional student.  And, of course, multiple foreign languages are taught at the college level.");
    	
    	//Mulva
    	mMarkers.addNewItem(44.444002, -88.068610, R.drawable.mulva, "Mulva Library", "The St. Norbert College experience is one of those rare events that profoundly affects the direction of a persons life. Its about learning. Its about community. Most of all, its about personal growth. The Miriam B. and James J. Mulva Library will accomplish all of those tenets, directly addressing the Colleges mission to provide an educational environment that is intellectually, spiritually and personally challenging.\n\nThe Miriam B. and James J. Mulva Library serves as the centerpiece and a primary entry point for the St. Norbert College campus. More than a mere depository for materials, it provides a convenient portal to the resources of the Information Age.\n\nSurprisingly, St. Norbert College has never had a dedicated library building in its more than 100 years of existence. From a small room in Main Hall to various locations in Boyle Hall, library space was simply found within existing buildings. The former Todd Wehr Library, which served the College from 1977 through June of 2009, was a remodeled dormitory. It was fashioned during a time when the internet did not yet exist.\n\nThe primary reason for a new campus library was the Colleges need to remain competitive in todays education marketplace. We understand that such a facility can be a deciding factor for prospective students and faculty as they weigh the benefits of becoming part of the St. Norbert community. Beyond this reality, our students deserve a state-of-the-art library capable of continuingeven expandingthe Colleges tradition of educational excellence.\n\nThe new library features a variety of gathering places for library patrons and 50 percent more materials storage, creating a modern sanctuary for knowledge. Most important, it offers a full range of technological capabilities needed by students in the 21st century, from wireless internet access for laptop users to multimedia-equipped meeting rooms.\n\nKnowledgeable staff and adequate on-site materials are taking on a greater importance as library users look for direction in the vast information landscape that is the internet.\n\nThe library marks the campuss western gateway, providing a major presence on the west side of Third Street and initiating the transition to the Colleges emerging western edge. It offers an attractive gathering point outside of the traditional campus core, advancing the Colleges master-plan goal of establishing a transitional Town and Gown zone with the De Pere community.");
    	
    	//Campus Center
    	mMarkers.addNewItem(44.443158, -88.065938, R.drawable.cc, "Campus Center", "The cultural and social heartbeat of the campus is the Ray Van Den Heuvel Family Campus Center, a renovated historical structure situated on the banks of the scenic Fox River.  The facility features indoor and outdoor seating for dining; a classic natural-light gymnasium reserved for non-varsity activities; workout rooms, fitness equipment, and common areas to study, catch a show, meet friends or just hang out.\n\nThe Campus Center hosts events throughout the summer as well, including the popular Knights on the Fox concert series on the rivers edge.");
    	
    	//Set up the tour screen
    	//Must run this AFTER adding markers, because it asks them for locations
    	setupTour();
    	
    }
    
    @Override
    protected void onPause() {
    	super.onPause();
    	//mLocationManager.removeUpdates(mLocationListener);
    }
    
    @Override
    protected void onResume() {
    	super.onResume();
    	//mLocationManager.requestLocationUpdates(mProvider, 2000, 2, mLocationListener);
    	
    }
    
    

	private void setupTour() {
		//Get handles for all the controls
		mTourCaption = (TextView)findViewById(R.id.tourcaption);
		mTourPrev = (Button) findViewById(R.id.tour_prev);
		mTourNext = (Button) findViewById(R.id.tour_next);
		mTourShowOnMap = (Button) findViewById(R.id.tour_goto);
		
		//Setup structures needed for proximity detection
		//Intents are used for interprocess communication
		mTourProximityIntent = new Intent("com.vromtr.tourproximity");
		mTourPendingIntent = PendingIntent.getBroadcast(this, -1, mTourProximityIntent, 0);
		
		//Register our handler to receive proximity alerts
		IntentFilter filter = new IntentFilter("com.vromtr.tourproximity");
		registerReceiver(new ProximityHandler(this), filter);
		
		//Set event handlers for all the buttons
		mTourPrev.setOnClickListener(new OnClickListener() {

			//Go to the previous point in the tour
			public void onClick(View v) {
				tourGoBack();
			}
			
		});
		
		mTourNext.setOnClickListener(new OnClickListener() {

			//Go to the previous point in the tour
			public void onClick(View v) {
				tourGoForward();
			}
			
		});
		
		mTourShowOnMap.setOnClickListener(new OnClickListener() {

			//Go to the previous point in the tour
			public void onClick(View v) {
				mMarkers.jumpToMarker(getCurrentDestination());
			}
			
		});
		
		//Populate the tour UI with meaningful data
		updateTour();
	}

	//Getter for which building we're going to
	protected String getCurrentDestination() {
		return mTourStops[mTourState];
	}

	//Update tour information
	//Main tab #3 UI and proximity detection
	private void updateTour() {
		//Update the "next step" caption
		mTourCaption.setText(mTourCaptions[mTourState]);
		
		//Add a star on the map!
		mMarkers.setNextDestination(mTourState);
		
		//Get the previous/next buttons on our form
		Button btnPrev = (Button)findViewById(R.id.tour_prev);
		Button btnNext = (Button)findViewById(R.id.tour_next);

		
		//If at beginning of tour, disable previous button
		btnPrev.setEnabled(mTourState != 0);

		
		//If at end of tour, disable next button
		btnNext.setEnabled(mTourState != mTourStops.length - 1);

		
		//Remove the old proximity detection if it's there
		if (mTourPendingIntent != null)
		{
			mLocationManager.removeProximityAlert(mTourPendingIntent);
		}
		
		//Get the location of the next building on the tour
		GeoPoint nextPoint = mMarkers.getLandmarkLocation(getCurrentDestination());
		double lat = ((double) nextPoint.getLatitudeE6())/1E6;
		double lng = ((double) nextPoint.getLongitudeE6())/1E6;
		float radius = 10f;	//Fire the proximity event when you get within this many meters of the destination
		
		//Add the new proximity alert
		mLocationManager.addProximityAlert(lat, lng, radius, -1, mTourPendingIntent);
		
		mTabHost.setCurrentTab(2);
		
	}
	
	//Go back to the previous point in the tour
	public void tourGoBack()
	{
		if (mTourState > 0)
		{
			mTourState--;
			updateTour();
		}
	}
	
	public void tourGoForward()
	{
		if (mTourState < mTourNumStates)
		{
			mTourState++;
			updateTour();
		}
	}

	private void setupGPS() {
		//Set up GPS code
    	

    	
    	//A "context" is something along the lines of an application handle
    	//We use the context to get the name by which we ask for a handle to the GPS hardware
    	String context = Context.LOCATION_SERVICE;
    	mLocationManager = (LocationManager) getSystemService(context);
    	
    	//Phones have multiple "location providers" - GPS, cell triangulation, guess from WiFi hotspots, randLatAndLong(), etc.
    	//Here we pick a location provider based on criteria we specify
    	Criteria criteria = new Criteria();
    	
    	criteria.setAccuracy(Criteria.ACCURACY_FINE);	//Pretty much means GPS.  "Fine" accuracy means stalker-approved proximity.
    	criteria.setAltitudeRequired(false);			//We don't need an altimeter
    	criteria.setBearingRequired(false);				//We don't really need to know which direction you're facing
    	criteria.setCostAllowed(true);					//Some cell companies charge for GPS use?!  Bastards.  There's a ring for them a few floors up from popes and usurers.
    	criteria.setPowerRequirement(Criteria.NO_REQUIREMENT);	//All the examples had POWER_LOW, but I care more about it finding the GPS hardware than battery life
    	//criteria.setPowerRequirement(Criteria.POWER_LOW);
    	
    	//Summon forth a location provider that matches our requirements!  (And is on!)
    	mProvider = mLocationManager.getBestProvider(criteria, true);
    	


    	mLocationListener = new LocationListener()
    	{

			public void onLocationChanged(Location location) {
				//Uncomment this line to keep the view centered on their current location
				//Could have this as a configurable option
				//updateWithNewLocation(location);
				
				//Inform our activity of the user's new location in meatspace
				setMyLocation(location);
				
			}

			public void onProviderDisabled(String provider) {
				// TODO Auto-generated method stub
				
			}

			public void onProviderEnabled(String provider) {
				// TODO Auto-generated method stub
				
			}

			public void onStatusChanged(String provider, int status,
					Bundle extras) {
				// TODO Auto-generated method stub
				
			}
    		
    	};
    	
    	//Register our location listener for updates.
    	//We'll be called with a new Location every time the phone  moves at least 2 meters
    	//but no more than once every 2000ms
    	mLocationManager.requestLocationUpdates(mProvider, 2000, 2, mLocationListener);
    	
    	mMyLocation = mLocationManager.getLastKnownLocation(mProvider);
    	
    	if (mMyLocation != null)
    	{
    		//jumpToMapLocation(mMyLocation);
    	}    	
    	
	}

	//Setter method for the user's current location in meatspace
	public void setMyLocation(Location newlocation)
	{
		mMyLocation = newlocation;
	}
	
	//Configure the MapView control used on Tab #1
    private void setupMapView() {
    	//An extra helper class we need to get graphics for the marker pins out of our application's resource
    	Resources r = getResources();
    	
    	/*
    	 * Construct our MapView
    	 */
    	
    	//Get a handle to our MapView control; store it.
    	mMapView = (MapView) findViewById(R.id.map_view);
    	
    	//Get the MapView's associated controller.
    	//Believe it or not, it's a helper class used for controlling the MapView
    	mMapController = mMapView.getController();
    	
    	//Set the zoom/display/visible controls of our MapView
    	mMapView.setSatellite(true);		//False gets regular map tiles
    	mMapController.setZoom(18);			//Highest zoom level available for SNC (and Brown County v.v')
    	
    	/*
    	 * Construct Overlay #1 - Compass and You Are Here dot 
    	 */
    	
    	
    	//Overlays are graphical layers drawn over the map control
    	//They intercept keypresses and allow us to annotate the map
    	//We're going to add a few to the MapView's list.
    	List<Overlay> overlays = mMapView.getOverlays();
    	
    	//The TapLocationsOverlay is a custom overlay class
    	//For development purposes it reads the lat/lon coordinates of wherever you tapped on the screen
    	//But, the base class is also responsible for drawing the compass and "You Are Here" dot.
    	TapLocationsOverlay myLocationOverlay = new TapLocationsOverlay(this, mMapView);
    	
    	//Turn on the compass and the You Are Here dot
    	myLocationOverlay.enableCompass();
    	myLocationOverlay.enableMyLocation();
    	

    	/*
    	 * Construct Overlay #2 - Interactive points of interest markers
    	 */
    	
    	//DynamicOverlay is a custom class derived from ItemizedOverlay
    	//It handles annotation and interacting with the marker pins.
    	//To instantiate one, we need a graphic to use for the pins, a context, and a pointer to our activity
    	mMarkers = new DynamicOverlay(r.getDrawable(R.drawable.rankalee), TabbedMaps.this, this);
    	
    	
    	//Start the program zoomed in on their last known location

    	
    	

    	//Place the overlays over our mapview
    	overlays.add(myLocationOverlay);
    	overlays.add(mMarkers);
	}
    
    
    
    
    
    

	//Builds the Points of Interest list on tab #2
    private void setupPointsOfInterestList() {
		//Grab and store a handle to the ListView on tab #2
        mPoiList = (ListView)findViewById(R.id.poilist);
        
        //Construct an array adapter.
        //This is a helper class used to take a sane collection class and force it into a UI element
        ArrayAdapter<String> aa = new ArrayAdapter<String>(this, android.R.layout.simple_list_item_1, pointsOfInterest);
        mPoiList.setAdapter(aa);
        
        //Add a click handler so our POI list is interactive
        mPoiList.setOnItemClickListener(new OnItemClickListener() {
        	
			public void onItemClick(AdapterView<?> _av, View _v, int _index,
					long arg3) {
				

				//The first entry is "Where Am I" and doesn't actually have a corresponding dialog
				if (_index == 0)
				{
					mMyLocation = mLocationManager.getLastKnownLocation(mProvider);
					jumpToMapLocation(mMyLocation);
				}
				
				//All other entries have an associated dialog box
				else
				{
					//Call our Activity's showDialog handler to display the appropriate information
					showDialog(_index);
					
				}
			}
        });
	}
    
    

	//Configure our activity's TabHost control, which is responsible for the tabs on the UI
    private void setupTabHost() {
		//Get and store a handle to our tabhost
    	mTabHost = (TabHost)findViewById(R.id.tabhost);
    	
    	//Must call this if we're interacting with it through findViewById
    	//People with sufficiently boring programs can avoid this by deriving a custom activity from TabHostActivity,
    	//where setup() is called in the base class's constructor
    	mTabHost.setup();
    	
    	//Add our UI tabs
    	mTabHost.addTab(mTabHost.newTabSpec("whereami").setIndicator("Where Am I").setContent(R.id.tab1view));
        mTabHost.addTab(mTabHost.newTabSpec("poi").setIndicator("Points of Interest").setContent(R.id.poilist));
        mTabHost.addTab(mTabHost.newTabSpec("tab_test3").setIndicator("Guided Tour").setContent(R.id.tourframe));
        
        //Switch to the Google Maps tab
        mTabHost.setCurrentTab(0);
	}
    
    
    
    


    
    @Override
	protected boolean isRouteDisplayed() {
		//Returns "true" if we're displaying driving directions
		//We're not, so return false.
		//MUST be implemented, or our MapView goes all runtime-exception-y on us.
		return false;
	}
	
	//SDK Reference says this is a "callback" method - not sure what that means in the context of Java; it doesn't behave like win32 callbacks
	//But, it's called in response to showDialog(int), and it knows to attach that dialog to the current activity
	//Using this method to display information about different points of interest
	protected Dialog onCreateDialog(int id) {
	    switch(id) {
	    case COFRIN_DIALOG:
	    	return mMarkers.getDialog("Cofrin Hall");
	    	
	    case BOYLE_DIALOG:
	    	return mMarkers.getDialog("Boyle Hall");
	    	
	    case WEHR_DIALOG:
	    	return mMarkers.getDialog("Todd Wehr Hall");
	    	
	    case MULVA_DIALOG:
	    	return mMarkers.getDialog("Mulva Library");
	    	
	    case BEMIS_DIALOG:
	    	return mMarkers.getDialog("Bemis International Center");
	    	
	    case CC_DIALOG:
	    	return mMarkers.getDialog("Campus Center");
	    	
	    default:
	        return null;
	    }
	}
	
	//Returns to the MapView tab and centers the view on POINT
	public void jumpToMapLocation(GeoPoint point)
	{
		mTabHost.setCurrentTab(0);
		mMapController.animateTo(point);
	}
	
	public void jumpToMapLocation(Location location)
	{
		//Convert to GeoPoints (millionths of degrees) used by MapView
		GeoPoint point = new GeoPoint( (int)(location.getLatitude() * 1E6), (int)(location.getLongitude() * 1E6));
		jumpToMapLocation(point);
	}
}

